@@ -1,6 +1,4 @@
 //
-// "$Id$"
-//
 // Valuator widget for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2016 by Bill Spitzak and others.
@@ -9,11 +7,11 @@
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 
@@ -103,7 +101,7 @@ void Fl_Valuator::handle_drag(double v) {
     value_ = v;
     value_damage();
     set_changed();
-    if (when() & FL_WHEN_CHANGED) do_callback();
+    if (when() & FL_WHEN_CHANGED) do_callback(FL_REASON_CHANGED);
   }
 }
 /** Called after an FL_WHEN_RELEASE event is received and before the callback. */
@@ -115,7 +113,7 @@ void Fl_Valuator::handle_release() {
     clear_changed();
     // now do the callback only if slider in new position or always is on:
     if (value_ != previous_value_ || when() & FL_WHEN_NOT_CHANGED) {
-      do_callback();
+      do_callback(FL_REASON_RELEASED);
     }
   }
 }
@@ -150,7 +148,7 @@ double Fl_Valuator::increment(double v, int n) {
 /**
   Uses internal rules to format the fields numerical value into
   the character array pointed to by the passed parameter.
-  
+
   The actual format used depends on the current step value. If
   the step value has been set to zero then a \%g format is used.
   If the step value is non-zero, then a \%.*f format is used,
@@ -192,7 +190,3 @@ int Fl_Valuator::format(char* buffer) {
   // MRS: THIS IS A HACK - RECOMMEND ADDING BUFFER SIZE ARGUMENT
   return snprintf(buffer, 128, "%.*f", c, v);
 }
-
-//
-// End of "$Id$".
-//
