@@ -1,6 +1,4 @@
 //
-// "$Id$"
-//
 // Filename extension routines for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
@@ -9,18 +7,17 @@
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
-// returns pointer to the last '.' or to the null if none:
+#include "Fl_System_Driver.H"
+#include <FL/Fl.H>
 
-#include <FL/filename.H>
-
-/** Gets the extensions of a filename.
+/** Gets the extension of a filename.
    \code
    #include <FL/filename.H>
    [..]
@@ -32,18 +29,35 @@
    \return a pointer to the extension (including '.') if any or NULL otherwise
  */
 const char *fl_filename_ext(const char *buf) {
+  return Fl::system_driver()->filename_ext(buf);
+}
+
+
+/**
+ \cond DriverDev
+ \addtogroup DriverDeveloper
+ \{
+ */
+
+/**
+ Finds a filename extension.
+
+ The default implementation assumes that the last `.` character separates
+ the extension from the basename of a file.
+
+ \see fl_filename_ext(const char*)
+ */
+const char *Fl_System_Driver::filename_ext(const char *buf) {
   const char *q = 0;
   const char *p = buf;
   for (p=buf; *p; p++) {
     if (*p == '/') q = 0;
-#if defined(WIN32) || defined(__EMX__) && !defined(__CYGWIN__)
-    else if (*p == '\\') q = 0;
-#endif
     else if (*p == '.') q = p;
   }
   return q ? q : p;
 }
 
-//
-// End of "$Id$".
-//
+/**
+ \}
+ \endcond
+ */
