@@ -1,6 +1,4 @@
 /*
- * "$Id: flstring.c 9573 2012-06-06 03:38:02Z fabien $"
- *
  * BSD string functions for the Fast Light Tool Kit (FLTK).
  *
  * Copyright 1998-2010 by Bill Spitzak and others.
@@ -9,11 +7,11 @@
  * the file "COPYING" which should have been included with this file.  If this
  * file is missing or damaged, see the license at:
  *
- *     http://www.fltk.org/COPYING.php
+ *     https://www.fltk.org/COPYING.php
  *
- * Please report all bugs and problems on the following page:
+ * Please see the following page on how to report bugs and issues:
  *
- *     http://www.fltk.org/str.php
+ *     https://www.fltk.org/bugs.php
  */
 
 #include "flstring.h"
@@ -23,12 +21,12 @@
  * 'fl_strlcat()' - Safely concatenate two strings.
  */
 
-size_t				/* O - Length of string */
-fl_strlcat(char       *dst,	/* O - Destination string */
-           const char *src,	/* I - Source string */
-	   size_t     size) {	/* I - Size of destination string buffer */
-  size_t	srclen;		/* Length of source string */
-  size_t	dstlen;		/* Length of destination string */
+size_t                          /* O - Length of string */
+fl_strlcat(char       *dst,     /* O - Destination string */
+           const char *src,     /* I - Source string */
+           size_t     size) {   /* I - Size of destination string buffer */
+  size_t        srclen;         /* Length of source string */
+  size_t        dstlen;         /* Length of destination string */
 
 
  /*
@@ -38,7 +36,7 @@ fl_strlcat(char       *dst,	/* O - Destination string */
   dstlen = strlen(dst);
   size   -= dstlen + 1;
 
-  if (!size) return (dstlen);	/* No room, return immediately... */
+  if (!size) return (dstlen);   /* No room, return immediately... */
 
  /*
   * Figure out how much room is needed...
@@ -58,38 +56,6 @@ fl_strlcat(char       *dst,	/* O - Destination string */
   return (dstlen + srclen);
 }
 
-
-/*
- * 'fl_strlcpy()' - Safely copy two strings.
- */
-
-size_t				/* O - Length of string */
-fl_strlcpy(char       *dst,	/* O - Destination string */
-           const char *src,	/* I - Source string */
-	   size_t      size) {	/* I - Size of destination string buffer */
-  size_t	srclen;		/* Length of source string */
-
-
- /*
-  * Figure out how much room is needed...
-  */
-
-  size --;
-
-  srclen = strlen(src);
-
- /*
-  * Copy the appropriate amount...
-  */
-
-  if (srclen > size) srclen = size;
-
-  memcpy(dst, src, srclen);
-  dst[srclen] = '\0';
-
-  return (srclen);
-}
-
 #define C_RANGE(c,l,r) ( (c) >= (l) && (c) <= (r) )
 
 /**
@@ -98,18 +64,14 @@ fl_strlcpy(char       *dst,	/* O - Destination string */
  */
 int fl_ascii_strcasecmp(const char *s, const char *t) {
   if (!s || !t) return (s==t ? 0 : (!s ? -1 : +1));
-  
+
   for(;*s && *t; s++,t++) {
     if (*s == *t) continue;
     if (*s < *t) {
       if ( (*s+0x20)!=*t || !C_RANGE(*s,'A','Z') ) return -1;
-    } else { 	/* (*s > *t) */
+    } else {    /* (*s > *t) */
       if ( (*s-0x20)!=*t || !C_RANGE(*s,'a','z') ) return +1;
     }
   }
   return (*s==*t) ? 0 : (*t ? -1 : +1);
 }
-
-/*
- * End of "$Id: flstring.c 9573 2012-06-06 03:38:02Z fabien $".
- */
