@@ -1,19 +1,17 @@
 //
-// "$Id: print_panel.cxx 10234 2014-08-21 12:18:32Z cand $"
-//
 // Print panel for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2017 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 //
@@ -25,11 +23,11 @@
 //
 // Todo:
 //
-//   -	Currently preferences can't be saved, and there are options that
-//	are not yet used for printing.
-//   -	This file can only be used as an include file in Fl_PS_Printer.cxx
-//   -	The use of static variables should be avoided.
-//   -	Probably much more ...
+//   -  Currently preferences can't be saved, and there are options that
+//      are not yet used for printing.
+//   -  This file can only be used as an include file in Fl_PS_Printer.cxx
+//   -  The use of static variables should be avoided.
+//   -  Probably much more ...
 //
 
 #include "print_panel.h"
@@ -38,8 +36,8 @@
 #include "../src/flstring.h"
 #include <FL/Fl_Preferences.H>
 #include <FL/Fl_Int_Input.H>
+#include <FL/fl_string_functions.h>
 
-static Fl_Preferences print_prefs(Fl_Preferences::USER, "fltk.org", "printers");
 static Fl_Double_Window *print_panel=(Fl_Double_Window *)0;
 static Fl_Group *print_panel_controls=(Fl_Group *)0;
 static Fl_Choice *print_choice=(Fl_Choice *)0;
@@ -57,7 +55,7 @@ static Fl_Int_Input *print_from=(Fl_Int_Input *)0;
 static Fl_Int_Input *print_to=(Fl_Int_Input *)0;
 static Fl_Spinner *print_copies=(Fl_Spinner *)0;
 
-static int print_start = 0;	// 1 if print_okay has been clicked
+static int print_start = 0;     // 1 if print_okay has been clicked
 
 static void cb_print_choice(Fl_Choice*, void*) {
   print_update_status();
@@ -120,6 +118,7 @@ static const Fl_Menu_Item menu_print_page_size[] = {
   {"B5", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
   {"Com10", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
   {"DL", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
+  {"Tabloid", 0,  0, 0, 0, FL_NORMAL_LABEL, 0, 14, 0},
  {0,0,0,0,0,0,0,0,0}
 };
 
@@ -225,6 +224,8 @@ static void cb_Save(Fl_Return_Button*, void*) {
   int val;
   const char *printer = (const char *)print_choice->menu()[print_choice->value()].user_data();
 
+  Fl_Preferences print_prefs(Fl_Preferences::CORE_USER, "fltk.org", "printers");
+
   snprintf(name, sizeof(name), "%s/page_size", printer == NULL ? "" : printer);
   print_prefs.set(name, print_page_size->value());
 
@@ -513,10 +514,11 @@ void print_cb(Fl_Return_Button *, void *) {
   print_panel->hide();
 }
 
-void print_load() {
+printing_style print_load() { // return whether SystemV or BSD printing style is used
   FILE *lpstat;
-  char line[1024], name[1024], *nptr, qname[2048], *qptr, defname[1024];
+  char line[1024], name[1024], *nptr, qname[2048], *qptr, defname[1024], *p;
   int i;
+  printing_style style = SystemV;
 
   if (print_choice->size() > 1) {
     for (i = 1; print_choice->text(i); i ++) {
@@ -527,12 +529,12 @@ void print_load() {
   print_choice->clear();
   print_choice->add(Fl_Printer::dialog_print_to_file, 0, 0, 0, FL_MENU_DIVIDER);
   print_choice->value(0);
-  
+
   print_start = 0;
 
   defname[0] = '\0';
-
-  if ((lpstat = popen("LC_MESSAGES=C LANG=C lpstat -p -d", "r")) != NULL) {
+// get names of all printers and of default one
+  if ((lpstat = popen("LC_MESSAGES=C LANG=C /bin/sh -c '(lpstat -p -d ) 2>&-'", "r")) != NULL) { // try first with SystemV printing system
     while (fgets(line, sizeof(line), lpstat)) {
       if (!strncmp(line, "printer ", 8) &&
           sscanf(line + 8, "%s", name) == 1) {
@@ -541,7 +543,7 @@ void print_load() {
         }
         *qptr = '\0';
 
-        print_choice->add(qname, 0, 0, (void *)strdup(name), 0);
+        print_choice->add(qname, 0, 0, (void *)fl_strdup(name), 0);
       } else if (!strncmp(line, "system default destination: ", 28)) {
         if (sscanf(line + 28, "%s", defname) != 1) defname[0] = '\0';
       }
@@ -549,17 +551,38 @@ void print_load() {
     pclose(lpstat);
   }
 
-  if (defname[0]) {
+  if (print_choice->size() == 2 && (lpstat = fopen("/etc/printcap", "r"))) { // try next with BSD printing system
+    while (fgets(line, sizeof(line),lpstat)) { // get names of all known printers
+      if (*line == '#' || (p = strchr(line, '|')) == NULL) continue;
+      *p = 0;
+      print_choice->add(line, 0, 0, (void *)fl_strdup(line), 0);
+      style = BSD;
+      *p = '|';
+      while (1) {
+        p = line + strlen(line) - 1;
+        if (*p == '\n' && p > line) p--;
+        if (*p != '\\') break;
+        if (fgets(line, sizeof(line),lpstat)==0) { /* ignore */ }
+      }
+    }
+    fclose(lpstat);
+    p = fl_getenv("PRINTER"); // get name of default printer
+    if (p == NULL) p = (char*)"lp";
+    strcpy(defname, p);
+  }
+
+  if (print_choice->size() > 2) print_choice->value(1);
+  if (defname[0]) { // select default printer in menu
     for (i = 1; print_choice->text(i); i ++) {
       if (!strcmp((char *)print_choice->menu()[i].user_data(), defname)) {
         print_choice->value(i);
         break;
       }
     }
-  } else if (print_choice->size() > 2) print_choice->value(1);
+  }
 
   print_update_status();
-
+  return style;
 } // print_load()
 
 void print_update_status() {
@@ -568,19 +591,28 @@ void print_update_status() {
   static char status[1024];
   const char *printer = (const char *)print_choice->menu()[print_choice->value()].user_data();
 
+  status[0] = 0;
   if (print_choice->value()) {
-    snprintf(command, sizeof(command), "lpstat -p '%s'", printer);
-    if ((lpstat = popen(command, "r")) != NULL) {
-      if (fgets(status, sizeof(status), lpstat)==0) { /* ignore */ }
-      pclose(lpstat);
-    } else strcpy(status, "printer status unavailable");
-  } else status[0] = '\0';
-
+    strcpy(status, "printer status unavailable");
+    snprintf(command, sizeof(command), "/bin/sh -c \"(lpstat -p '%s' ) 2>&-\" ", printer); // try first with SystemV printing system
+    if ((lpstat = popen(command, "r")) !=  NULL) {
+      if (fgets(status, sizeof(status), lpstat) == 0) { // if no reply
+        pclose(lpstat);
+        snprintf(command, sizeof(command), "lpq -P%s 2>&-", printer); // try next with BSD printing system
+        if ((lpstat = popen(command, "r")) !=  NULL) {
+          if (fgets(status, sizeof(status), lpstat)==0) { /* ignore */ }
+        }
+      }
+    pclose(lpstat);
+    }
+  }
   print_status->label(status);
 
   char name[1024];
   int val;
 
+  Fl_Preferences print_prefs(Fl_Preferences::CORE_USER, "fltk.org", "printers");
+
   snprintf(name, sizeof(name), "%s/page_size", printer == NULL ? "" : printer);
   print_prefs.get(name, val, 1);
   print_page_size->value(val);
@@ -589,7 +621,3 @@ void print_update_status() {
   print_prefs.get(name, val, 0);
   print_output_mode[val]->setonly();
 }
-
-//
-// End of "$Id: print_panel.cxx 10234 2014-08-21 12:18:32Z cand $".
-//
