@@ -1,5 +1,4 @@
-/* "$Id: utf8Wrap.c 10248 2014-08-23 08:41:58Z cand $"
- *
+/*
  * Author: Jean-Marc Lienher ( http://oksid.ch )
  * Copyright 2000-2003 by O'ksi'D.
  *
@@ -7,24 +6,27 @@
  * the file "COPYING" which should have been included with this file.  If this
  * file is missing or damaged, see the license at:
  *
- *     http://www.fltk.org/COPYING.php
+ *     https://www.fltk.org/COPYING.php
  *
- * Please report all bugs and problems on the following page:
+ * Please see the following page on how to report bugs and issues:
  *
- *     http://www.fltk.org/str.php
+ *     https://www.fltk.org/bugs.php
  */
 
 /*
  * X11 UTF-8 text drawing functions.
+ *
+ * This file is compiled and linked only for X11 w/o Xft.
  */
-#if !defined(WIN32) && !defined(__APPLE__)
 
 #include "../Xutf8.h"
 #include <X11/Xlib.h>
+#include <FL/fl_string_functions.h>  // fl_strdup()
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include "../utf8_internal.h"
 
 /* External auto generated functions : */
 #include "ucs2fontmap.c"
@@ -35,7 +37,7 @@
  */
 
 /* The ARM header files have a bug by not taking into account that ARM cpu
- * likes pacing to 4 bytes. This little trick defines our own version of
+ * likes padding to 4 bytes. This little trick defines our own version of
  * XChar2b which does not have this problem
  */
 
@@ -56,8 +58,8 @@ Fl_XChar2b;
 /*********************************************************************/
 static int
 get_font_list(
-	const char	*base_font_name_list,
-	char 		***flist) {
+        const char      *base_font_name_list,
+        char            ***flist) {
   const char *ptr;
   const char *p;
   int nb;
@@ -99,7 +101,7 @@ get_font_list(
 
 /*********************************************************************/
 /** get the font name used as encoding for "fontspecific" encoding  **/
-/** (mainly used for adobe-symbol and adobe-zapfdingbats)	    **/
+/** (mainly used for adobe-symbol and adobe-zapfdingbats)           **/
 /*********************************************************************/
 static int
 font_spec_enc(char *font) {
@@ -124,12 +126,12 @@ font_spec_enc(char *font) {
 
 
 /*********************************************************************/
-/** get the sub range of a iso10646-1 font			    **/
+/** get the sub range of a iso10646-1 font                          **/
 /*********************************************************************/
 static void
-get_range(const char	*enc,
-	  int 		*min,
-	  int		*max) {
+get_range(const char    *enc,
+          int           *min,
+          int           *max) {
 
   const char *ptr = enc;
   const char *ptr1;
@@ -154,12 +156,12 @@ get_range(const char	*enc,
 }
 
 /*********************************************************************/
-/** get the internal encoding number of each fonts 		    **/
+/** get the internal encoding number of each fonts                  **/
 /*********************************************************************/
 static int *
-get_encodings(char	**font_name_list,
-	      int 	*ranges,
-	      int 	nb_font) {
+get_encodings(char      **font_name_list,
+              int       *ranges,
+              int       nb_font) {
 
   int *font_encoding_list;
   int i;
@@ -181,16 +183,16 @@ get_encodings(char	**font_name_list,
     }
     while (ptr && *ptr) {
       if (*ptr == '-') {
-	ec++;
-	if (ec == 13) {
-	  font_encoding_list[i] = encoding_number(ptr + 1);
-	  if (font_encoding_list[i] == 0) {
-	    get_range(ptr + 1,
-		      ranges + i * 2,
-		      ranges + i * 2 + 1);
-	  }
-	  break;
-	}
+        ec++;
+        if (ec == 13) {
+          font_encoding_list[i] = encoding_number(ptr + 1);
+          if (font_encoding_list[i] == 0) {
+            get_range(ptr + 1,
+                      ranges + i * 2,
+                      ranges + i * 2 + 1);
+          }
+          break;
+        }
       }
       ptr++;
     }
@@ -201,11 +203,11 @@ get_encodings(char	**font_name_list,
 }
 
 /*********************************************************************/
-/** find the first font which matches the name and load it.	    **/
+/** find the first font which matches the name and load it.         **/
 /*********************************************************************/
 static XFontStruct *
 find_best_font(Display  *dpy,
-	       char     **name) {
+               char     **name) {
 
   char **list;
   int cnt;
@@ -214,7 +216,7 @@ find_best_font(Display  *dpy,
   list = XListFonts(dpy, *name, 1, &cnt);
   if (cnt && list) {
     free(*name);
-    *name = strdup(list[0]);
+    *name = fl_strdup(list[0]);
     s = XLoadQueryFont(dpy, *name);
     XFreeFontNames(list);
     return s;
@@ -223,17 +225,13 @@ find_best_font(Display  *dpy,
 }
 
 /*********************************************************************/
-/** load all fonts 						    **/
+/** load all fonts                                                  **/
 /*********************************************************************/
 static void
-load_fonts(Display 	   *dpy,
-	   XUtf8FontStruct *font_set) {
+load_fonts(Display         *dpy,
+           XUtf8FontStruct *font_set) {
 
-  int i;
-  /* char **list; */
-
-  i = 0;
-  /* list = NULL; */
+  int i = 0;
 
   font_set->fonts = (XFontStruct**) malloc(sizeof(XFontStruct*) *
                                            font_set->nb_font);
@@ -253,10 +251,10 @@ load_fonts(Display 	   *dpy,
     if (fnt) {
       font_set->fid = fnt->fid;
       if (fnt->ascent > font_set->ascent) {
-	font_set->ascent = fnt->ascent;
+        font_set->ascent = fnt->ascent;
       }
       if (fnt->descent > font_set->descent) {
-	font_set->descent = fnt->descent;
+        font_set->descent = fnt->descent;
       }
     } else {
       free(font_set->font_name_list[i]);
@@ -274,18 +272,18 @@ load_fonts(Display 	   *dpy,
     if (font_set->font_name_list[i]) {
       int j;
       for (j = 0; j < i; j++) {
-	if (font_set->font_name_list[j] &&
-	    font_set->encodings[j] ==
-	    font_set->encodings[i] &&
-	    font_set->ranges[2*j] ==
-	    font_set->ranges[2*i] &&
-	    font_set->ranges[(2*j)+1] &&
-	    font_set->ranges[(2*i)+1]) {
-	  XFreeFont(dpy, font_set->fonts[i]);
-	  free(font_set->font_name_list[i]);
-	  font_set->font_name_list[i] = NULL;
-	  font_set->fonts[i] = 0;
-	}
+        if (font_set->font_name_list[j] &&
+            font_set->encodings[j] ==
+            font_set->encodings[i] &&
+            font_set->ranges[2*j] ==
+            font_set->ranges[2*i] &&
+            font_set->ranges[(2*j)+1] &&
+            font_set->ranges[(2*i)+1]) {
+          XFreeFont(dpy, font_set->fonts[i]);
+          free(font_set->font_name_list[i]);
+          font_set->font_name_list[i] = NULL;
+          font_set->fonts[i] = 0;
+        }
       }
     }
   }
@@ -293,11 +291,11 @@ load_fonts(Display 	   *dpy,
 
 /*********************************************************************/
 /** Creates an array of XFontStruct acording to the comma separated **/
-/** list of fonts. XLoad all fonts.				    **/
+/** list of fonts. XLoad all fonts.                                 **/
 /*********************************************************************/
 XUtf8FontStruct *
 XCreateUtf8FontStruct(Display    *dpy,
-		      const char *base_font_name_list) {
+                      const char *base_font_name_list) {
 
   XUtf8FontStruct *font_set;
 
@@ -308,7 +306,7 @@ XCreateUtf8FontStruct(Display    *dpy,
   }
 
   font_set->nb_font = get_font_list(base_font_name_list,
-				    &font_set->font_name_list);
+                                    &font_set->font_name_list);
 
   if (font_set->nb_font < 1) {
     free(font_set);
@@ -322,29 +320,29 @@ XCreateUtf8FontStruct(Display    *dpy,
 
 
 /*****************************************************************************/
-/** draw a Right To Left UTF-8 string using multiple fonts as needed.	    **/
+/** draw a Right To Left UTF-8 string using multiple fonts as needed.       **/
 /*****************************************************************************/
 void
-XUtf8DrawRtlString(Display 		*display,
-		   Drawable 		d,
-		   XUtf8FontStruct 	*font_set,
-		   GC 			gc,
-		   int 			x,
-		   int 			y,
-		   const char		*string,
-		   int 			num_bytes) {
+XUtf8DrawRtlString(Display              *display,
+                   Drawable             d,
+                   XUtf8FontStruct      *font_set,
+                   GC                   gc,
+                   int                  x,
+                   int                  y,
+                   const char           *string,
+                   int                  num_bytes) {
 
-  int 		*encodings;	/* encodings array */
-  XFontStruct 	**fonts;	/* fonts array */
-  Fl_XChar2b 	buf[128];	/* drawing buffer */
-  Fl_XChar2b	*ptr;		/* pointer to the drawing buffer */
-  int 		fnum;		/* index of the current font in the fonts array*/
-  int 		i;		/* current byte in the XChar2b buffer */
-  int 		first;		/* first valid font index */
-  int 		last_fnum;	/* font index of the previous char */
-  int 		nb_font;	/* quantity of fonts in the font array */
-  char 		glyph[2];	/* byte1 and byte2 value of the UTF-8 char */
-  int		*ranges;	/* sub range of iso10646 */
+  int           *encodings;     /* encodings array */
+  XFontStruct   **fonts;        /* fonts array */
+  Fl_XChar2b    buf[128];       /* drawing buffer */
+  Fl_XChar2b    *ptr;           /* pointer to the drawing buffer */
+  int           fnum;           /* index of the current font in the fonts array*/
+  int           i;              /* current byte in the XChar2b buffer */
+  int           first;          /* first valid font index */
+  int           last_fnum;      /* font index of the previous char */
+  int           nb_font;        /* quantity of fonts in the font array */
+  char          glyph[2];       /* byte1 and byte2 value of the UTF-8 char */
+  int           *ranges;        /* sub range of iso10646 */
 
   nb_font = font_set->nb_font;
 
@@ -370,7 +368,7 @@ XUtf8DrawRtlString(Display 		*display,
   last_fnum = fnum;
 
   while (num_bytes > 0) {
-    int 	 ulen;   /* byte length of the UTF-8 char */
+    int          ulen;   /* byte length of the UTF-8 char */
     unsigned int ucs;    /* Unicode value of the UTF-8 char */
     unsigned int no_spc; /* Spacing char equivalent of a non-spacing char */
 
@@ -383,7 +381,7 @@ XUtf8DrawRtlString(Display 		*display,
       ptr = buf + 128;
     }
 
-    ulen = XFastConvertUtf8ToUcs((unsigned char*)string, num_bytes, &ucs);
+    ulen = XFastConvertUtf8ToUcs((const unsigned char*)string, num_bytes, &ucs);
 
     if (ulen < 1) ulen = 1;
 
@@ -397,10 +395,10 @@ XUtf8DrawRtlString(Display 		*display,
     fnum = first;
     while (fnum < nb_font) {
       if (fonts[fnum] && ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
-	if (encodings[fnum] != 0 ||
-	    (ucs >= ranges[fnum * 2] && ucs <= ranges[fnum * 2 + 1])) {
-	  break;
-	}
+        if (encodings[fnum] != 0 ||
+            ((int)ucs >= ranges[fnum * 2] && (int)ucs <= ranges[fnum * 2 + 1])) {
+          break;
+        }
       }
       fnum++;
     }
@@ -421,7 +419,7 @@ XUtf8DrawRtlString(Display 		*display,
       (*ptr).byte1 = glyph[0];
       (*ptr).byte2 = glyph[1];
       if (no_spc) {
-	x += XTextWidth16(fonts[fnum], ptr, 1);
+        x += XTextWidth16(fonts[fnum], ptr, 1);
       }
     } else {
       ptr--;
@@ -443,28 +441,28 @@ XUtf8DrawRtlString(Display 		*display,
 
 
 /*****************************************************************************/
-/** draw an UTF-8 string using multiple fonts as needed.		    **/
+/** draw an UTF-8 string using multiple fonts as needed.                    **/
 /*****************************************************************************/
 void
-XUtf8DrawString(Display 	*display,
-	        Drawable 	d,
+XUtf8DrawString(Display         *display,
+                Drawable        d,
                 XUtf8FontStruct *font_set,
-	        GC 		gc,
-	        int 		x,
-	        int 		y,
-	        const char	*string,
-                int 		num_bytes) {
+                GC              gc,
+                int             x,
+                int             y,
+                const char      *string,
+                int             num_bytes) {
 
-  int 		*encodings; /* encodings array */
-  XFontStruct 	**fonts;    /* fonts array */
-  Fl_XChar2b 	buf[128];   /* drawing buffer */
-  int 		fnum;       /* index of the current font in the fonts array*/
-  int 		i;          /* current byte in the XChar2b buffer */
-  int 		first;      /* first valid font index */
-  int 		last_fnum;  /* font index of the previous char */
-  int 		nb_font;    /* quantity of fonts in the font array */
-  char 		glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
-  int		*ranges;    /* sub range of iso10646 */
+  int           *encodings; /* encodings array */
+  XFontStruct   **fonts;    /* fonts array */
+  Fl_XChar2b    buf[128];   /* drawing buffer */
+  int           fnum;       /* index of the current font in the fonts array*/
+  int           i;          /* current byte in the XChar2b buffer */
+  int           first;      /* first valid font index */
+  int           last_fnum;  /* font index of the previous char */
+  int           nb_font;    /* quantity of fonts in the font array */
+  char          glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
+  int           *ranges;    /* sub range of iso10646 */
 
   nb_font = font_set->nb_font;
 
@@ -488,7 +486,7 @@ XUtf8DrawString(Display 	*display,
   last_fnum = fnum;
 
   while (num_bytes > 0) {
-    int 	 ulen;   /* byte length of the UTF-8 char */
+    int          ulen;   /* byte length of the UTF-8 char */
     unsigned int ucs;    /* Unicode value of the UTF-8 char */
     unsigned int no_spc; /* Spacing char equivalent of a non-spacing char */
 
@@ -500,7 +498,7 @@ XUtf8DrawString(Display 	*display,
       i = 0;
     }
 
-    ulen = XFastConvertUtf8ToUcs((unsigned char*)string, num_bytes, &ucs);
+    ulen = XFastConvertUtf8ToUcs((const unsigned char*)string, num_bytes, &ucs);
 
     if (ulen < 1) ulen = 1;
 
@@ -514,11 +512,11 @@ XUtf8DrawString(Display 	*display,
     fnum = first;
     while (fnum < nb_font) {
       if (fonts[fnum] && ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
-	if (encodings[fnum] != 0 ||
-	    (ucs >= ranges[fnum * 2] &&
-	    ucs <= ranges[fnum * 2 + 1])) {
-	  break;
-	}
+        if (encodings[fnum] != 0 ||
+            ((int)ucs >= ranges[fnum * 2] &&
+             (int)ucs <= ranges[fnum * 2 + 1])) {
+          break;
+        }
       }
       fnum++;
     }
@@ -538,7 +536,7 @@ XUtf8DrawString(Display 	*display,
       (*buf).byte1 = glyph[0];
       (*buf).byte2 = glyph[1];
       if (no_spc) {
-	x -= XTextWidth16(fonts[fnum], buf, 1);
+        x -= XTextWidth16(fonts[fnum], buf, 1);
       }
     } else {
       (*(buf + i)).byte1 = glyph[0];
@@ -562,26 +560,26 @@ XUtf8DrawString(Display 	*display,
 /*****************************************************************************/
 void
 XUtf8_measure_extents(
-        Display         	*display,
-        Drawable        	d,
+        Display                 *display,
+        Drawable                d,
         XUtf8FontStruct  *font_set,
-        GC              	gc,
-        int             	*xx,     /* x-offset from origin */
-        int             	*yy,     /* y-offset from origin */
-        int             	*ww,     /* overall inked width  */
-        int             	*hh,     /* maximum inked height */
-        const char      	*string, /* text to measure */
-        int             	num_bytes) {
-  int 		*encodings; /* encodings array */
-  XFontStruct 	**fonts;    /* fonts array */
-  Fl_XChar2b 	buf[128];   /* drawing buffer */
-  int 		fnum;       /* index of the current font in the fonts array*/
-  int 		i;          /* current byte in the XChar2b buffer */
-  int 		first;      /* first valid font index */
-  int 		last_fnum;  /* font index of the previous char */
-  int 		nb_font;    /* quantity of fonts in the font array */
-  char 		glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
-  int		*ranges;    /* sub range of iso10646 */
+        GC                      gc,
+        int                     *xx,     /* x-offset from origin */
+        int                     *yy,     /* y-offset from origin */
+        int                     *ww,     /* overall inked width  */
+        int                     *hh,     /* maximum inked height */
+        const char              *string, /* text to measure */
+        int                     num_bytes) {
+  int           *encodings; /* encodings array */
+  XFontStruct   **fonts;    /* fonts array */
+  Fl_XChar2b    buf[128];   /* drawing buffer */
+  int           fnum;       /* index of the current font in the fonts array*/
+  int           i;          /* current byte in the XChar2b buffer */
+  int           first;      /* first valid font index */
+  int           last_fnum;  /* font index of the previous char */
+  int           nb_font;    /* quantity of fonts in the font array */
+  char          glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
+  int           *ranges;    /* sub range of iso10646 */
 
   int wd = 0; /* accumulates the width of the text */
   int ht = 0; /* used to find max height in text */
@@ -617,7 +615,7 @@ XUtf8_measure_extents(
   last_fnum = fnum;
 
   while (num_bytes > 0) {
-    int 	 ulen;   /* byte length of the UTF-8 char */
+    int          ulen;   /* byte length of the UTF-8 char */
     unsigned int ucs;    /* Unicode value of the UTF-8 char */
     unsigned int no_spc; /* Spacing char equivalent of a non-spacing char */
 
@@ -633,7 +631,7 @@ XUtf8_measure_extents(
       i = 0;
     }
 
-    ulen = XFastConvertUtf8ToUcs((unsigned char*)string, num_bytes, &ucs);
+    ulen = XFastConvertUtf8ToUcs((const unsigned char*)string, num_bytes, &ucs);
 
     if (ulen < 1) ulen = 1;
 
@@ -647,11 +645,11 @@ XUtf8_measure_extents(
     fnum = first;
     while (fnum < nb_font) {
       if (fonts[fnum] && ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
-	if (encodings[fnum] != 0 ||
-	    (ucs >= ranges[fnum * 2] &&
-	    ucs <= ranges[fnum * 2 + 1])) {
-	  break;
-	}
+        if (encodings[fnum] != 0 ||
+            ((int)ucs >= ranges[fnum * 2] &&
+             (int)ucs <= ranges[fnum * 2 + 1])) {
+          break;
+        }
       }
       fnum++;
     }
@@ -675,7 +673,7 @@ XUtf8_measure_extents(
       (*buf).byte1 = glyph[0];
       (*buf).byte2 = glyph[1];
       if (no_spc) {
-	wd -= XTextWidth16(fonts[fnum], buf, 1);
+        wd -= XTextWidth16(fonts[fnum], buf, 1);
       }
     } else {
       (*(buf + i)).byte1 = glyph[0];
@@ -703,24 +701,24 @@ XUtf8_measure_extents(
 
 
 /*****************************************************************************/
-/** returns the pixel width of a UTF-8 string				    **/
+/** returns the pixel width of a UTF-8 string                               **/
 /*****************************************************************************/
 int
-XUtf8TextWidth(XUtf8FontStruct 	*font_set,
-	       const char 	*string,
-	       int 		num_bytes) {
+XUtf8TextWidth(XUtf8FontStruct  *font_set,
+               const char       *string,
+               int              num_bytes) {
 
-  int		x;
-  int 		*encodings; /* encodings array */
-  XFontStruct 	**fonts;    /* fonts array */
-  Fl_XChar2b 	buf[128];   /* drawing buffer */
-  int 		fnum;       /* index of the current font in the fonts array*/
-  int 		i;          /* current byte in the XChar2b buffer */
-  int 		first;      /* first valid font index */
-  int 		last_fnum;  /* font index of the previous char */
-  int 		nb_font;    /* quantity of fonts in the font array */
-  char 		glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
-  int		*ranges;    /* sub range of iso10646 */
+  int           x;
+  int           *encodings; /* encodings array */
+  XFontStruct   **fonts;    /* fonts array */
+  Fl_XChar2b    buf[128];   /* drawing buffer */
+  int           fnum;       /* index of the current font in the fonts array*/
+  int           i;          /* current byte in the XChar2b buffer */
+  int           first;      /* first valid font index */
+  int           last_fnum;  /* font index of the previous char */
+  int           nb_font;    /* quantity of fonts in the font array */
+  char          glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
+  int           *ranges;    /* sub range of iso10646 */
 
   nb_font = font_set->nb_font;
   x = 0;
@@ -746,7 +744,7 @@ XUtf8TextWidth(XUtf8FontStruct 	*font_set,
   last_fnum = fnum;
 
   while (num_bytes > 0) {
-    int 	 ulen;   /* byte length of the UTF-8 char */
+    int          ulen;   /* byte length of the UTF-8 char */
     unsigned int ucs;    /* Unicode value of the UTF-8 char */
     unsigned int no_spc; /* Spacing char equivalent of a non-spacing char */
 
@@ -756,7 +754,7 @@ XUtf8TextWidth(XUtf8FontStruct 	*font_set,
       i = 0;
     }
 
-    ulen = XFastConvertUtf8ToUcs((unsigned char*)string, num_bytes, &ucs);
+    ulen = XFastConvertUtf8ToUcs((const unsigned char*)string, num_bytes, &ucs);
 
     if (ulen < 1) ulen = 1;
 
@@ -772,11 +770,11 @@ XUtf8TextWidth(XUtf8FontStruct 	*font_set,
     fnum = first;
     while (fnum < nb_font) {
       if (fonts[fnum] && ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
-	if (encodings[fnum] != 0 ||
-		(ucs >= ranges[fnum * 2] &&
-		ucs <= ranges[fnum * 2 + 1])) {
-	  break;
-	}
+        if (encodings[fnum] != 0 ||
+            ((int)ucs >= ranges[fnum * 2] &&
+             (int)ucs <= ranges[fnum * 2 + 1])) {
+          break;
+        }
       }
       fnum++;
     }
@@ -794,8 +792,8 @@ XUtf8TextWidth(XUtf8FontStruct 	*font_set,
       (*buf).byte1 = glyph[0];
       (*buf).byte2 = glyph[1];
       if (no_spc) {
-	/* go back to draw the non-spacing char over the previous char */
-	x -= XTextWidth16(fonts[fnum], buf, 1);
+        /* go back to draw the non-spacing char over the previous char */
+        x -= XTextWidth16(fonts[fnum], buf, 1);
       }
     } else {
       (*(buf + i)).byte1 = glyph[0];
@@ -816,24 +814,20 @@ XUtf8TextWidth(XUtf8FontStruct 	*font_set,
 /**  get the X font and glyph ID of a UCS char                              **/
 /*****************************************************************************/
 int
-XGetUtf8FontAndGlyph(XUtf8FontStruct  *font_set,
-		     unsigned int     ucs,
-		     XFontStruct      **fnt,
-		     unsigned short   *id) {
+fl_XGetUtf8FontAndGlyph(XUtf8FontStruct  *font_set,
+                        unsigned int     ucs,
+                        XFontStruct      **fnt,
+                        unsigned short   *id) {
 
-  /* int             x; */
   int             *encodings; /* encodings array */
   XFontStruct     **fonts;    /* fonts array */
   int             fnum;       /* index of the current font in the fonts array*/
-  /* int             i; */         /* current byte in the XChar2b buffer */
   int             first;      /* first valid font index */
-  /* int             last_fnum; */ /* font index of the previous char */
   int             nb_font;    /* quantity of fonts in the font array */
-  char 		  glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
+  char            glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
   int             *ranges;    /* sub range of iso10646 */
 
   nb_font = font_set->nb_font;
-  /* x = 0; */
 
   if (nb_font < 1) {
     /* there is no font in the font_set :-( */
@@ -843,7 +837,6 @@ XGetUtf8FontAndGlyph(XUtf8FontStruct  *font_set,
   ranges = font_set->ranges;
   fonts = font_set->fonts;
   encodings = font_set->encodings;
-  /* i = 0; */
   fnum = 0;
 
   while(fnum < nb_font && !fonts[fnum]) fnum++;
@@ -853,19 +846,17 @@ XGetUtf8FontAndGlyph(XUtf8FontStruct  *font_set,
   }
 
   first = fnum;
-  /* last_fnum = fnum; */
 
   /*
-   * find the first encoding which can be used to
-   * draw the glyph
+   * find the first encoding which can be used to draw the glyph
    */
   fnum = first;
   while (fnum < nb_font) {
     if (fonts[fnum] && ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
       if (encodings[fnum] != 0 ||
-          (ucs >= ranges[fnum * 2] &&
-	  ucs <= ranges[fnum * 2 + 1])) {
-	break;
+          ((int)ucs >= ranges[fnum * 2] &&
+           (int)ucs <= ranges[fnum * 2 + 1])) {
+        break;
       }
     }
     fnum++;
@@ -884,23 +875,21 @@ XGetUtf8FontAndGlyph(XUtf8FontStruct  *font_set,
 }
 
 /*****************************************************************************/
-/** returns the pixel width of a UCS char				    **/
+/** returns the pixel width of a UCS char                                   **/
 /*****************************************************************************/
 int
 XUtf8UcsWidth(XUtf8FontStruct  *font_set,
-	      unsigned int     ucs) {
+              unsigned int     ucs) {
 
-  int		x;
-  int 		*encodings; /* encodings array */
-  XFontStruct 	**fonts;    /* fonts array */
-  Fl_XChar2b 	buf[8];     /* drawing buffer */
-  int 		fnum;       /* index of the current font in the fonts array*/
-  /*int 		i;*/          /* current byte in the XChar2b buffer */
-  int 		first;      /* first valid font index */
-  /*int 		last_fnum;*/ /* font index of the previous char */
-  int 		nb_font;    /* quantity of fonts in the font array */
-  char 		glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
-  int		*ranges;    /* sub range of iso10646 */
+  int           x;
+  int           *encodings; /* encodings array */
+  XFontStruct   **fonts;    /* fonts array */
+  Fl_XChar2b    buf[8];     /* drawing buffer */
+  int           fnum;       /* index of the current font in the fonts array*/
+  int           first;      /* first valid font index */
+  int           nb_font;    /* quantity of fonts in the font array */
+  char          glyph[2];   /* byte1 and byte2 value of the UTF-8 char */
+  int           *ranges;    /* sub range of iso10646 */
   unsigned int  no_spc;
 
   nb_font = font_set->nb_font;
@@ -914,7 +903,6 @@ XUtf8UcsWidth(XUtf8FontStruct  *font_set,
   ranges = font_set->ranges;
   fonts = font_set->fonts;
   encodings = font_set->encodings;
-  /* i = 0; */
   fnum = 0;
 
   while(fnum < nb_font && !fonts[fnum]) fnum++;
@@ -924,7 +912,6 @@ XUtf8UcsWidth(XUtf8FontStruct  *font_set,
   }
 
   first = fnum;
-  /* last_fnum = fnum; */
 
   no_spc = XUtf8IsNonSpacing(ucs);
   if (no_spc) ucs = no_spc;
@@ -936,10 +923,10 @@ XUtf8UcsWidth(XUtf8FontStruct  *font_set,
   fnum = first;
   while (fnum < nb_font) {
     if (fonts[fnum] &&
-	ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
-      if (encodings[fnum] != 0 || (ucs >= ranges[fnum * 2] &&
-	  ucs <= ranges[fnum * 2 + 1])) {
-	break;
+        ucs2fontmap(glyph, ucs, encodings[fnum]) >= 0) {
+      if (encodings[fnum] != 0 || ((int)ucs >= ranges[fnum * 2] &&
+          (int)ucs <= ranges[fnum * 2 + 1])) {
+        break;
       }
     }
     fnum++;
@@ -961,17 +948,17 @@ XUtf8UcsWidth(XUtf8FontStruct  *font_set,
 }
 
 /*****************************************************************************/
-/** draw an UTF-8 string and clear the background.	 		    **/
+/** draw an UTF-8 string and clear the background.                          **/
 /*****************************************************************************/
 void
 XUtf8DrawImageString(Display         *display,
-		     Drawable        d,
-		     XUtf8FontStruct *font_set,
-		     GC              gc,
-		     int             x,
-		     int             y,
-		     const char      *string,
-		     int             num_bytes) {
+                     Drawable        d,
+                     XUtf8FontStruct *font_set,
+                     GC              gc,
+                     int             x,
+                     int             y,
+                     const char      *string,
+                     int             num_bytes) {
 
   /* FIXME: must be improved ! */
   int w;
@@ -984,7 +971,7 @@ XUtf8DrawImageString(Display         *display,
   w = XUtf8TextWidth(font_set, string, num_bytes);
 
   XGetGCValues(display, gc,
-	       GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
+               GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
 
   function = xgcv.function;
   fill_style = xgcv.fill_style;
@@ -997,10 +984,10 @@ XUtf8DrawImageString(Display         *display,
   xgcv.fill_style = FillSolid;
 
   XChangeGC(display, gc,
-	    GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
+            GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
 
   XFillRectangle(display, d, gc, x, y - font_set->ascent,
-	         (unsigned)w, (unsigned)(font_set->ascent + font_set->descent));
+                 (unsigned)w, (unsigned)(font_set->ascent + font_set->descent));
 
   xgcv.function = function;
   xgcv.foreground = foreground;
@@ -1008,7 +995,7 @@ XUtf8DrawImageString(Display         *display,
   xgcv.fill_style = fill_style;
 
   XChangeGC(display, gc,
-	    GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
+            GCFunction|GCForeground|GCBackground|GCFillStyle, &xgcv);
 
   XUtf8DrawString(display, d, font_set, gc, x, y, string, num_bytes);
 }
@@ -1017,15 +1004,15 @@ XUtf8DrawImageString(Display         *display,
 /** free the XFontSet and others things created by XCreateUtf8FontSet       **/
 /*****************************************************************************/
 void
-XFreeUtf8FontStruct(Display 	    *dpy,
-		    XUtf8FontStruct *font_set) {
+XFreeUtf8FontStruct(Display         *dpy,
+                    XUtf8FontStruct *font_set) {
 
   int i;
   i = 0;
   while (i < font_set->nb_font) {
     if (font_set->fonts[i]) {
-	XFreeFont(dpy, font_set->fonts[i]);
-	free(font_set->font_name_list[i]);
+        XFreeFont(dpy, font_set->fonts[i]);
+        free(font_set->font_name_list[i]);
     }
     i++;
   }
@@ -1035,9 +1022,3 @@ XFreeUtf8FontStruct(Display 	    *dpy,
   free(font_set->encodings);
   free(font_set);
 }
-
-#endif /* X11 only */
-
-/*
- *  End of "$Id: utf8Wrap.c 10248 2014-08-23 08:41:58Z cand $".
- */
