@@ -1,6 +1,4 @@
 //
-// "$Id: doublebuffer.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
-//
 // Double-buffering test program for the Fast Light Tool Kit (FLTK).
 //
 // This demo shows how double buffering helps, by drawing the
@@ -19,11 +17,11 @@
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 #include <FL/Fl.H>
@@ -71,22 +69,20 @@ void bad_draw(int w,int h,int which) {
 }
 
 class single_blink_window : public Fl_Single_Window {
-  void draw() {bad_draw(w(),h(),0); draw_child(*child(0));}
+  void draw() FL_OVERRIDE {bad_draw(w(),h(),0); draw_child(*child(0));}
 public:
   single_blink_window(int x, int y,int w,int h,const char *l)
     : Fl_Single_Window(x,y,w,h,l) {resizable(this);}
 };
 
 class double_blink_window : public Fl_Double_Window {
-  void draw() {bad_draw(w(),h(),1); draw_child(*child(0));}
+  void draw() FL_OVERRIDE {bad_draw(w(),h(),1); draw_child(*child(0));}
 public:
   double_blink_window(int x, int y, int w,int h,const char *l)
     : Fl_Double_Window(x,y,w,h,l) {resizable(this);}
 };
 
 int main(int argc, char **argv) {
-  if (!Fl::visual(FL_DOUBLE))
-    printf("Xdbe not supported, faking double buffer with pixmaps.\n");
   Fl_Window w01(420,420,"Fl_Single_Window"); w01.box(FL_FLAT_BOX);
   single_blink_window w1(10,10,400,400,"Fl_Single_Window");
   w1.box(FL_FLAT_BOX); w1.color(FL_BLACK); //w1.position(100,200);
@@ -113,7 +109,3 @@ int main(int argc, char **argv) {
   w2.show();
   return Fl::run();
 }
-
-//
-// End of "$Id: doublebuffer.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
-//
