@@ -1,19 +1,16 @@
 //
-// "$Id: fullscreen.cxx 9605 2012-06-14 15:09:46Z manolo $"
 //
-
-//
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2025 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
 // file is missing or damaged, see the license at:
 //
-//     http://www.fltk.org/COPYING.php
+//     https://www.fltk.org/COPYING.php
 //
-// Please report all bugs and problems on the following page:
+// Please see the following page on how to report bugs and issues:
 //
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 //
@@ -67,7 +64,7 @@
 #include <FL/Fl_Gl_Window.H>
 
 class shape_window : public Fl_Gl_Window {
-  void draw();
+  void draw() FL_OVERRIDE;
 public:
   int sides;
   shape_window(int x,int y,int w,int h,const char *l=0);
@@ -79,19 +76,19 @@ Fl_Gl_Window(x,y,w,h,l) {
 }
 
 void shape_window::draw() {
-  printf("drawing size %d %d\n",w(),h());
+  // fprintf(stderr, "drawing size %d %d\n", w(), h());
   if (!valid()) {
     valid(1);
-//  printf("init\n");
+    // fprintf(stderr, "init\n");
     glLoadIdentity();
-    glViewport(0,0,w(),h());
+    glViewport(0,0,pixel_w(),pixel_h());
   }
   glClear(GL_COLOR_BUFFER_BIT);
-  glColor3f(.5,.6,.7);
+  glColor3f(.5f, .6f, .7f);
   glBegin(GL_POLYGON);
   for (int j = 0; j < sides; j ++) {
     double ang = j*2*M_PI/sides;
-    glVertex3f(cos(ang),sin(ang),0);
+    glVertex3f((GLfloat)cos(ang), (GLfloat)sin(ang), 0);
   }
   glEnd();
 }
@@ -101,7 +98,7 @@ void shape_window::draw() {
 #include <FL/fl_draw.H>
 
 class shape_window : public Fl_Window {
-  void draw();
+  void draw() FL_OVERRIDE;
 public:
   int sides;
   shape_window(int x,int y,int w,int h,const char *l=0);
@@ -124,25 +121,31 @@ void shape_window::draw() {
 
 class fullscreen_window : public Fl_Single_Window {
   public:
-  fullscreen_window(int W, int H, const char *t=0);
-  int handle (int e);
-  Fl_Toggle_Light_Button *b3;
-
+  fullscreen_window(int W, int H, const char *t=0) : Fl_Single_Window(W, H, t) {}
+  void resize(int x, int y, int w, int h) FL_OVERRIDE;
+  Fl_Toggle_Light_Button *border_button;
+  Fl_Toggle_Light_Button *maximize_button;
+  Fl_Toggle_Light_Button *fullscreen_button;
+  Fl_Toggle_Light_Button *allscreens_button;
 };
 
-fullscreen_window::fullscreen_window(int W, int H, const char *t) : Fl_Single_Window(W, H, t) { 
 
+void after_resize(fullscreen_window *win) {
+  if (win->maximize_active()) win->maximize_button->set();
+  else win->maximize_button->clear();
+  win->maximize_button->redraw();
+  if (win->fullscreen_active()) win->fullscreen_button->set();
+  else win->fullscreen_button->clear();
+  win->fullscreen_button->redraw();
 }
 
-int fullscreen_window::handle(int e) {
-  if (e == FL_FULLSCREEN) {
-    printf("Received FL_FULLSCREEN event\n");
-    b3->value(fullscreen_active());
-  }
-  if (Fl_Single_Window::handle(e)) return 1;
-  return 0;
+
+void fullscreen_window::resize(int x, int y, int w, int h) {
+  Fl_Single_Window::resize(x,y,w,h);
+  Fl::add_timeout(0, (Fl_Timeout_Handler)after_resize, this);
 }
 
+
 void sides_cb(Fl_Widget *o, void *p) {
   shape_window *sw = (shape_window *)p;
   sw->sides = int(((Fl_Slider *)o)->value());
@@ -159,35 +162,80 @@ void double_cb(Fl_Widget *o, void *p) {
 void double_cb(Fl_Widget *, void *) {}
 #endif
 
-void border_cb(Fl_Widget *o, void *p) {
-  Fl_Window *w = (Fl_Window *)p;
-  int d = ((Fl_Button *)o)->value();
+
+void border_cb(Fl_Button *b, Fl_Window *w) {
+  int d = b->value();
   w->border(d);
-#if defined(WIN32) || defined(__APPLE__)
-  int wx = w->x(), wy = w->y();
-  w->hide(); w->show();
-  w->position(wx, wy);
-#endif
+  // border change may have been refused (e.g. with fullscreen window)
+  if ((int)w->border() != d) b->value(w->border());
 }
 
-int px,py,pw,ph;
-Fl_Button *border_button;
-void fullscreen_cb(Fl_Widget *o, void *p) {
+
+void maximize_cb(Fl_Button *b, Fl_Window *w) {
+  if (w->fullscreen_active()) {
+    b->value(1 - b->value());
+    return;
+  }
+  if (w->maximize_active()) {
+    w->un_maximize();
+  } else {
+    w->maximize();
+  }
+}
+
+
+void fullscreen_cb(Fl_Button *b, Fl_Window *w) {
+  if (w->maximize_active()) {
+    b->value(1 - b->value());
+    return;
+  }
+  if (b->value()) {
+    w->fullscreen();
+  } else {
+    w->fullscreen_off();
+  }
+}
+
+void allscreens_cb(Fl_Widget *o, void *p) {
   Fl_Window *w = (Fl_Window *)p;
   int d = ((Fl_Button *)o)->value();
   if (d) {
-    px = w->x();
-    py = w->y();
-    pw = w->w();
-    ph = w->h();
-    w->fullscreen();
-    w->override();
-#ifndef WIN32 // update our border state in case border was turned off
-    border_button->value(w->border());
-#endif
+    int top, bottom, left, right;
+    int top_y, bottom_y, left_x, right_x;
+
+    int sx, sy, sw, sh;
+
+    top = bottom = left = right = 0;
+
+    Fl::screen_xywh(sx, sy, sw, sh, 0);
+    top_y = sy;
+    bottom_y = sy + sh;
+    left_x = sx;
+    right_x = sx + sw;
+
+    for (int i = 1;i < Fl::screen_count();i++) {
+      Fl::screen_xywh(sx, sy, sw, sh, i);
+      if (sy < top_y) {
+        top = i;
+        top_y = sy;
+      }
+      if ((sy + sh) > bottom_y) {
+        bottom = i;
+        bottom_y = sy + sh;
+      }
+      if (sx < left_x) {
+        left = i;
+        left_x = sx;
+      }
+      if ((sx + sw) > right_x) {
+        right = i;
+        right_x = sx + sw;
+      }
+    }
+
+    w->fullscreen_screens(top, bottom, left, right);
   } else {
-    //w->fullscreen_off(px,py,pw,ph);
-    w->fullscreen_off();
+    w->fullscreen_screens(-1, -1, -1, -1);
   }
 }
 
@@ -197,19 +245,21 @@ void update_screeninfo(Fl_Widget *b, void *p) {
     char line[128];
     browser->clear();
 
-    sprintf(line, "Main screen work area: %dx%d@%d,%d", Fl::w(), Fl::h(), Fl::x(), Fl::y());
+    snprintf(line, sizeof(line), "Main screen work area: %dx%d@%d,%d", Fl::w(), Fl::h(), Fl::x(), Fl::y());
     browser->add(line);
     Fl::screen_work_area(x, y, w, h);
-    sprintf(line, "Mouse screen work area: %dx%d@%d,%d", w, h, x, y);
+    snprintf(line, sizeof(line), "Mouse screen work area: %dx%d@%d,%d", w, h, x, y);
     browser->add(line);
     for (int n = 0; n < Fl::screen_count(); n++) {
-	int x, y, w, h;
-	Fl::screen_xywh(x, y, w, h, n);
-	sprintf(line, "Screen %d: %dx%d@%d,%d", n, w, h, x, y);
-	browser->add(line);
-	Fl::screen_work_area(x, y, w, h, n);
-	sprintf(line, "Work area %d: %dx%d@%d,%d", n, w, h, x, y);
-	browser->add(line);
+        int x, y, w, h;
+        float dpih, dpiv;
+        Fl::screen_xywh(x, y, w, h, n);
+        Fl::screen_dpi(dpih, dpiv, n);
+        snprintf(line, sizeof(line), "Screen %d: %dx%d@%d,%d DPI:%.1fx%.1f scale:%.2f", n, w, h, x, y, dpih, dpiv, Fl::screen_scale(n));
+        browser->add(line);
+        Fl::screen_work_area(x, y, w, h, n);
+        snprintf(line, sizeof(line), "Work area %d: %dx%d@%d,%d", n, w, h, x, y);
+        browser->add(line);
     }
 }
 
@@ -219,7 +269,7 @@ void exit_cb(Fl_Widget *, void *) {
   exit(0);
 }
 
-#define NUMB 7
+#define NUMB 9
 
 int twowindow = 0;
 int initfull = 0;
@@ -231,26 +281,27 @@ int arg(int, char **argv, int &i) {
 
 int main(int argc, char **argv) {
 
+  Fl::use_high_res_GL(1);
   int i=0;
   if (Fl::args(argc,argv,i,arg) < argc)
     Fl::fatal("Options are:\n -2 = 2 windows\n -f = startup fullscreen\n%s",Fl::help);
 
-  fullscreen_window window(400,400+30*NUMB); window.end();
+  fullscreen_window window(460,400+30*NUMB); window.end();
 
   shape_window sw(10,10,window.w()-20,window.h()-30*NUMB-120);
-
+  sw.set_visible(); // necessary because sw is not a child of window
 #if HAVE_GL
   sw.mode(FL_RGB);
 #endif
 
   Fl_Window *w;
-  if (twowindow) {	// make it's own window
+  if (twowindow) {      // make its own window
     sw.resizable(&sw);
     w = &sw;
-    window.set_modal();	// makes controls stay on top when fullscreen pushed
+    window.set_modal(); // makes controls stay on top when fullscreen pushed
     argc--;
     sw.show();
-  } else {		// otherwise make a subwindow
+  } else {              // otherwise make a subwindow
     window.add(sw);
     window.resizable(&sw);
     w = &window;
@@ -274,14 +325,21 @@ int main(int argc, char **argv) {
   Fl_Input i1(50,y,window.w()-60,30, "Input");
   y+=30;
 
-  Fl_Toggle_Light_Button b2(50,y,window.w()-60,30,"Border");
-  b2.callback(border_cb,w);
-  b2.set();
-  border_button = &b2;
+  window.border_button = new Fl_Toggle_Light_Button(50,y,window.w()-60,30,"Border");
+  window.border_button->callback((Fl_Callback*)border_cb,w);
+  window.border_button->set();
   y+=30;
 
-  window.b3 = new Fl_Toggle_Light_Button(50,y,window.w()-60,30,"FullScreen");
-  window.b3->callback(fullscreen_cb,w);
+  window.fullscreen_button = new Fl_Toggle_Light_Button(50,y,window.w()-60,30,"FullScreen");
+  window.fullscreen_button->callback((Fl_Callback*)fullscreen_cb,w);
+  y+=30;
+
+  window.maximize_button = new Fl_Toggle_Light_Button(50,y,window.w()-60,30,"Maximize");
+  window.maximize_button->callback((Fl_Callback*)maximize_cb,w);
+  y+=30;
+
+  window.allscreens_button = new Fl_Toggle_Light_Button(50,y,window.w()-60,30,"All Screens");
+  window.allscreens_button->callback(allscreens_cb,w);
   y+=30;
 
   Fl_Button eb(50,y,window.w()-60,30,"Exit");
@@ -291,19 +349,15 @@ int main(int argc, char **argv) {
   Fl_Browser *browser = new Fl_Browser(50,y,window.w()-60,100);
   update_screeninfo(0, browser);
   y+=100;
-  
+
   Fl_Button update(50,y,window.w()-60,30,"Update");
   update.callback(update_screeninfo, browser);
   y+=30;
 
-  if (initfull) {window.b3->set(); window.b3->do_callback();}
+  if (initfull) {window.fullscreen_button->set(); window.fullscreen_button->do_callback();}
 
   window.end();
   window.show(argc,argv);
-  
+
   return Fl::run();
 }
-
-//
-// End of "$Id: fullscreen.cxx 9605 2012-06-14 15:09:46Z manolo $".
-//
