@@ -1,28 +1,17 @@
 //
-// "$Id: symbols.cxx 7903 2010-11-28 21:06:39Z matt $"
-//
 // Symbol test program for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+//     https://www.fltk.org/COPYING.php
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Please see the following page on how to report bugs and issues:
 //
-// Please report all bugs and problems on the following page:
-//
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
 #include <stdio.h>
@@ -39,7 +28,7 @@ int N = 0;
 #define W 70
 #define H 70
 #define ROWS 6
-#define COLS 6
+#define COLS 7
 
 Fl_Double_Window *window;
 Fl_Value_Slider *orientation;
@@ -55,15 +44,15 @@ void slider_cb(Fl_Widget *, void *) {
     if ( l && *l == '@' ) {                       // all children with '@'
       l ++;
       if ( wc->box() == FL_NO_BOX ) {             // ascii legend?
-        if (val&&sze) sprintf(buf, "@@%+d%d%s", sze, val, l);
-        else if (val) sprintf(buf, "@@%d%s", val, l);
-        else if (sze) sprintf(buf, "@@%+d%s", sze, l);
-        else          sprintf(buf, "@@%s", l);
+        if (val&&sze) snprintf(buf, sizeof(buf), "@@%+d%d%s", sze, val, l);
+        else if (val) snprintf(buf, sizeof(buf), "@@%d%s", val, l);
+        else if (sze) snprintf(buf, sizeof(buf), "@@%+d%s", sze, l);
+        else          snprintf(buf, sizeof(buf), "@@%s", l);
       } else {                                    // box with symbol
-        if (val&&sze) sprintf(buf, "@%+d%d%s", sze, val, l);
-        else if (val) sprintf(buf, "@%d%s", val, l);
-        else if (sze) sprintf(buf, "@%+d%s", sze, l);
-        else          sprintf(buf, "@%s", l);
+        if (val&&sze) snprintf(buf, sizeof(buf), "@%+d%d%s", sze, val, l);
+        else if (val) snprintf(buf, sizeof(buf), "@%d%s", val, l);
+        else if (sze) snprintf(buf, sizeof(buf), "@%+d%s", sze, l);
+        else          snprintf(buf, sizeof(buf), "@%s", l);
       }
       wc->copy_label(buf);
     }
@@ -78,7 +67,7 @@ void bt(const char *name) {
   N++;
   x = x*W+10;
   y = y*H+10;
-  sprintf(buf, "@%s", name);
+  snprintf(buf, sizeof(buf), "@%s", name);
   Fl_Box *a = new Fl_Box(x,y,W-20,H-20);
   a->box(FL_NO_BOX);
   a->copy_label(buf);
@@ -130,6 +119,8 @@ bt("@refresh");
 bt("@reload");
 bt("@undo");
 bt("@redo");
+bt("@import");
+bt("@export");
 
   orientation = new Fl_Value_Slider(
     (int)(window->w()*.05+.5), window->h()-40,
@@ -153,7 +144,3 @@ bt("@redo");
   window->show(argc,argv);
   return Fl::run();
 }
-
-//
-// End of "$Id: symbols.cxx 7903 2010-11-28 21:06:39Z matt $".
-//
